@@ -1,5 +1,5 @@
 from django import forms
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 
 from localcosmos_server.models import APP_USER_ROLES
@@ -12,4 +12,4 @@ class AppUserRoleForm(forms.Form):
 
 
 class SearchAppUserForm(forms.Form):
-    search_user = forms.CharField(label=_('Search user'), required=False)
+    search_user = forms.CharField(label=_('Search users'), required=False)
