@@ -2,8 +2,9 @@ from django.conf import settings
 from django.db import models
 from django.contrib.auth.models import AbstractUser, UserManager
 from django.contrib.postgres.fields import JSONField
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 from django.urls import reverse
+from django.db import transaction
 
 
 from .taxonomy.generic import ModelWithRequiredTaxon
@@ -65,6 +66,41 @@ class LocalcosmosUser(AbstractUser):
 
     objects = LocalcosmosUserManager()
 
+    # do not alter the delete method
+    def delete(self, using=None, keep_parents=False):
+        if settings.LOCALCOSMOS_OPEN_SOURCE == True:
+            super().delete(using=using, keep_parents=keep_parents)
+        else:
+            # localcosmos.org uses django-tenants
+            from django_tenants.utils import schema_context
+            from lcsite.models import Tenant
+            
+            user_id = self.pk
+
+            # using transactions because multiple schemas can refer to the same
+            # user ID as FK references!
+            with transaction.atomic():
+
+                deleted = False
+                
+                # delete user and all of its data across tenants
+                for tenant in Tenant.objects.all().exclude(schema_name='public'):
+                    with schema_context(tenant.schema_name):
+                        
+                        super().delete(using=using, keep_parents=keep_parents)
+                        # reassign the ID because delete() sets it to None
+                        self.pk = user_id
+
+                        deleted = True
+
+                
+                if deleted == False:
+                    
+                    # deleting from public schema is not necessary, it happens on the first schema-specific deletion
+                    with schema_context('public'):
+                        super().delete()
+            
+
     class Meta:
         unique_together = ('email',)
 
@@ -88,14 +124,14 @@ class UserClients(models.Model):
 
 
 '''
-    an App is a webapp which is loaded by an index.html file
+    App
+    - an App is a webapp which is loaded by an index.html file
     - Apps are served by nginx or apache
 '''
 class AppManager(models.Manager):
 
     def create(self, name, primary_language, uid):
 
-        # circular import
         app = self.model(
             name=name,
             primary_language=primary_language,
@@ -105,6 +141,7 @@ class AppManager(models.Manager):
         app.save()
 
         return app
+    
         
 class App(models.Model):
     uuid = models.UUIDField(default=uuid.uuid4, unique=True, editable=False)
@@ -114,6 +151,7 @@ class App(models.Model):
     uid = models.CharField(max_length=255, unique=True, editable=False)
 
     # automatically download app updates when you click "publish" on localcosmos.org
+    # this feature is not implemented yet
     auto_update = models.BooleanField(default=True)
 
     primary_language = models.CharField(max_length=15)
@@ -123,16 +161,20 @@ class App(models.Model):
     # online content uses this to load a preview on the open source installation
     url = models.URLField(null=True)
 
-    # download the currently released apk here
+    # url for downloading the currently released apk
     apk_url = models.URLField(null=True)
 
-    # download currently released ipa here
+    # url for downloading the currently released ipa
+    # as of 2019 this does not make any sense, because apple does not support ad-hoc installations
+    # for companies < 100 employees
     ipa_url = models.URLField(null=True)
 
-    # download current webapp for installing on the private server
+    # COMMERCIAL ONLY
+    # url for downloading the current webapp for installing on the private server
     pwa_zip_url = models.URLField(null=True)
-    
-    # for version comparisons, version for apk, ipa and webapp
+
+    # COMMERCIAL ONLY ?
+    # for version comparisons, version of the app in the appkit, version of apk, ipa and webapp might differ
     published_version = models.IntegerField(null=True)
 
     # an asbolute path on disk to a folder containing a www folder with static index.html file
@@ -193,16 +235,36 @@ class App(models.Model):
         return '{0}{1}{2}/preview/www/'.format(domain.domain, settings.APP_KIT_PREVIEW_URL, self.uid)
 
 
-    # read app settings from disk, online_content
-    # preview=True is for commercial installation only
-    def get_settings(self, preview=True):
+    def get_installed_app_path(self, app_state):
 
         if settings.LOCALCOSMOS_OPEN_SOURCE == True:
-            preview = False
-        
-        root = self.published_version_path
-        if preview == True:
+            app_state = 'published'
+
+        if app_state == 'published':
+            root = self.published_version_path
+
+            # on the first build, there is no published_version_path, but only a review_version_path
+            # the "review apk" is exactly the same as the later "published apk",
+            # so fall back to review settings if no published settings are available
+            if root == None and settings.LOCALCOSMOS_OPEN_SOURCE == False:
+                root = self.review_version_path
+
+        elif app_state == 'preview':
             root = self.preview_version_path
+
+        elif app_state == 'review':
+            root= self.review_version_path
+
+        else:
+            raise ValueError('Invalid app_state: {0}'.format(app_state))
+        
+        return root
+
+    # read app settings from disk, online_content
+    # app_state=='preview' or app_state=='review' are for commercial installation only
+    def get_settings(self, app_state='preview'):
+
+        root = self.get_installed_app_path(app_state)
             
         settings_json_path = os.path.join(root, 'settings.json')
 
@@ -213,9 +275,10 @@ class App(models.Model):
 
 
     # read app features from disk, only published apps
-    # preview=True is for commercial installation only
-    def get_features(self, preview=True):
-        root = self.published_version_path
+    # app_state=='preview' or app_state=='review' are for commercial installation only
+    # used eg by AppTaxonSearch.py
+    def get_features(self, app_state='preview'):
+        root = self.get_installed_app_path(app_state)
         
         features_json_path = os.path.join(root, 'features.json')
 
@@ -226,18 +289,10 @@ class App(models.Model):
 
     # api_settings is used by localcosmos_server.api, eg to determine allow_anonymous_observations
     # api_settings only contains settings which make sense for the api (need_to_know basis)
-    # review=True is for commercial installation only
-    def get_api_settings(self, review=False):
-
-        if settings.LOCALCOSMOS_OPEN_SOURCE == True:
-            review = False
+    # app_state='review' is for commercial installation only
+    def get_api_settings(self, app_state='published'):
         
-        root = self.published_version_path
-
-        # on the first build, there is no published_version_path, but only a review_version_path
-        # the "review apk" es exactly the same as the later "published apk", so fall back to review settings if no published settings are available
-        if review == True or root == None:
-            root = self.review_version_path
+        root = self.get_installed_app_path(app_state)
 
         api_settings_json_path = os.path.join(root, 'api', 'settings.json')
 
@@ -258,23 +313,18 @@ class App(models.Model):
 
     ##############################################################################################################
     # theme
-    def get_installed_app_path(self):
-        if self.preview_version_path:
-            return self.preview_version_path
 
-        return self.published_version_path
-
-    def get_theme_path(self, preview=True):
+    def get_theme_path(self, app_state='preview'):
         
-        app_settings = self.get_settings(preview=preview)
+        app_settings = self.get_settings(app_state=app_state)
         
         theme_name = app_settings['THEME']
-        installed_app_path = self.get_installed_app_path()
+        installed_app_path = self.get_installed_app_path(app_state)
         return os.path.join(installed_app_path, 'themes', theme_name)
 
-    def get_theme_config(self, preview=True):
+    def get_theme_config(self, app_state='preview'):
 
-        theme_path = self.get_theme_path(preview=preview)
+        theme_path = self.get_theme_path(app_state=app_state)
         theme_config_path = os.path.join(theme_path, 'config.json')
 
         with open(theme_config_path, 'r') as theme_config_file:
@@ -284,31 +334,45 @@ class App(models.Model):
         
     ###############################################################################################################
     # online content specific
-    
-    def get_online_content_templates_path(self, preview=True):
-        app_theme_path = self.get_theme_path(preview=preview)
+    # on LOCALCOSMOS_OPEN_SOURCE==True app_state is always 'published'
+    # on LOCALCOSMOS_OPEN_SOURCE==False app_state is always 'preview'
+
+    def get_online_content_app_state(self):
+
+        if settings.LOCALCOSMOS_OPEN_SOURCE == True:
+            return 'published'
+
+        return 'preview'
+
+    # preview or published, depending on LOCALCOSMOS_OPEN_SOURCE
+    def get_online_content_templates_path(self):
+        
+        app_state = self.get_online_content_app_state()
+        
+        app_theme_path = self.get_theme_path(app_state=app_state)
         return os.path.join(app_theme_path, 'online_content', 'templates')
 
 
     # return the online_content specific theme settings
     # called from online_content.api.views and online_content.models.TemplateContentFlagsManager
-    def get_online_content_settings(self, preview=True):
-        theme_config = self.get_theme_config(preview=preview)
+    def get_online_content_settings(self):
+
+        app_state = self.get_online_content_app_state()
+        
+        theme_config = self.get_theme_config(app_state=app_state)
 
         oc_settings = theme_config['online_content']
         return oc_settings
     
-    # return a list of available templates, always use the live version, used by eg online_content.forms.CreateTemplateContentForm
+    # return a list of available templates, always use the preview version,
+    # used by eg online_content.forms.CreateTemplateContentForm
     # eg displays a selection in the AppAdmin
     def get_online_content_templates(self, template_type):
 
-        if self.published_version:
-            preview = False
-        else:
-            preview = True
+        app_state = self.get_online_content_app_state()
             
-        templates_path = os.path.join(self.get_online_content_templates_path(preview=preview), template_type)
-        oc_settings = self.get_online_content_settings(preview=preview)
+        templates_path = os.path.join(self.get_online_content_templates_path(), template_type)
+        oc_settings = self.get_online_content_settings()
         language = self.primary_language
 
         templates = []
@@ -326,7 +390,7 @@ class App(models.Model):
         return templates
 
 
-    def get_online_content_template(self, template_name, preview=True):
+    def get_online_content_template(self, template_name):
         # return an instance of Template
         # Template(contents, origin, origin.template_name, self.engine,)
         # contents is the content of the .html file
@@ -334,7 +398,9 @@ class App(models.Model):
         # engine is a template engine
         # Template can be instantiated directly, ony with contents
 
-        theme_path = self.get_theme_path(preview=preview)
+        app_state = self.get_online_content_app_state()
+
+        theme_path = self.get_theme_path(app_state=app_state)
         online_content_path = os.path.join(theme_path, 'online_content')
 
         template_path = os.path.join(online_content_path, 'templates', template_name)
