@@ -1,6 +1,6 @@
 from django.db import models
 from django.conf import settings
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext_lazy as _
 from django.contrib.contenttypes.models import ContentType
 from django.template.defaultfilters import slugify
 from django import forms
@@ -37,10 +37,12 @@ from .widgets import MultiContentWidget
     - page
     - feature (can be displayed on several pages via include, may not extend a base)
 '''
+
 TEMPLATE_TYPES = (
     ('page', _('Page')), # pages can have flags like "header", "footer" etc
     ('feature', _('Feature')), # e.g. newsbox - can be displayed on several pages
 )
+
 class TemplateContentManager(models.Manager):
 
     def create(self, creator, app, language, draft_title, draft_navigation_link_name, template_name,
@@ -69,13 +71,13 @@ class TemplateContent(models.Model):
     objects = TemplateContentManager()
 
     def get_theme_settings(self):
-        return self.app.get_online_content_settings(preview=True)
+        return self.app.get_online_content_settings()
 
     def verbose_template_name(self, language_code=None):
         if not language_code:
             language_code = self.app.primary_language
             
-        theme_settings = self.app.get_online_content_settings(preview=True)
+        theme_settings = self.app.get_online_content_settings()
 
         verbose_name = self.template_name
 
@@ -112,6 +114,8 @@ class TemplateContent(models.Model):
         else:
             return None
 
+
+    # might be obsolete
     def validate(self, app):
 
         result = {
@@ -162,10 +166,10 @@ class TemplateContent(models.Model):
         else:
             languages = [language]
 
-        # ltc.translation_ready is not set to True vy the user if there is only one language
+        # ltc.translation_ready is not set to True by the user if there is only one language
         # skip the check if the "translation" exists and also skip the check if the user has set
         # translation_ready to True, which is not the case because there is only a "publish" button
-        # in this case and no "ready for translation" button
+        # in this case (only 1 language) and no "ready for translation" button
         if not secondary_languages:
             ltc = LocalizedTemplateContent.objects.filter(template_content=self, language=primary_language).first()
             publication_errors += ltc.translation_complete()
@@ -174,7 +178,6 @@ class TemplateContent(models.Model):
         # set by the user when he has finished translating
         else:
 
-
             for language_code in languages:
 
                 # translation_complete checks two things:
@@ -342,60 +345,6 @@ class LocalizedTemplateContent(models.Model):
 
     objects = LocalizedTemplateContentManager()        
 
-    '''
-    publish a LocalizedTemplateContent and all its MicroContents
-    '''
-    def publish(self):
-        # set title
-        self.published_title = self.draft_title
-        self.navigation_link_name = self.draft_navigation_link_name
-
-        if self.published_version != self.draft_version:
-
-            self.published_version = self.draft_version
-            self.published_at = timezone.now()
-
-        self.save(published=True)
-        
-
-    def save(self, *args, **kwargs):
-
-        # indicates, if the save() command came from self.publish
-        published = kwargs.pop('published', False)
-        
-        # if only a new preview token is needed skip the publication protocol
-        disallow_new_version = kwargs.pop('disallow_new_version', False)
-
-        # first, check the slug - new slug if the title really changed
-        slug_base = LocalizedTemplateContent.objects.generate_slug_base(self.draft_title)
-        if not self.slug.startswith(slug_base):
-            old_slug = self.slug
-            self.slug = LocalizedTemplateContent.objects.generate_slug(self.draft_title)
-            new_slug = self.slug
-
-            # add to SlugTrail - make old links still work
-            slug_trail = SlugTrail(
-                old_slug = old_slug,
-                new_slug = new_slug,
-            )
-
-            slug_trail.save()
-
-        
-
-        if disallow_new_version == False and published == False:
-
-            # the localized_template_content has already been published. start new version
-            if self.published_version == self.draft_version:
-                self.draft_version += 1
-                self.translation_ready = False
-
-        super().save(*args, **kwargs)
-
-
-    def get_text_microcontent(self, microcontent_type):
-        return TextMicroContent.objects.filter(template_content=self.template_content,
-                                               microcontent_type=microcontent_type)
 
     '''
     - if the language is the primary language, check if all required fields are presend
@@ -409,7 +358,7 @@ class LocalizedTemplateContent(models.Model):
 
         # avoid circular import
         from .parser import TemplateParser
-        parser = TemplateParser(self.get_template())
+        parser = TemplateParser(self.template_content.app, self.template_content, self.get_template())
 
         cms_tags = parser.parse()
 
@@ -437,7 +386,8 @@ class LocalizedTemplateContent(models.Model):
             for tag in cms_tags:
 
                 # language independant - fetch all existing drafts
-                draft_microcontents = tag.Model.objects.filter(template_content=self.template_content, microcontent_type=tag.microcontent_type)
+                draft_microcontents = tag.Model.objects.filter(template_content=self.template_content,
+                                                               microcontent_type=tag.microcontent_type)
 
                 # check all contents, including all 'multi' 
                 for draft_microcontent in draft_microcontents:
@@ -479,8 +429,8 @@ class LocalizedTemplateContent(models.Model):
         return self.template_content.get_template()
 
     # improvement could be: read urls.js from the app and construct the url accordingly
-    def in_app_link(self, preview=False):
-        if preview:
+    def in_app_link(self, app_state='published'):
+        if app_state == 'preview':
             in_app_link = '/online-content/%s/%s/' %(self.slug, self.preview_token)
         else:
             in_app_link = '/online-content/%s/' %(self.slug)
@@ -488,6 +438,63 @@ class LocalizedTemplateContent(models.Model):
         return in_app_link
 
 
+    '''
+    publish a LocalizedTemplateContent and all its MicroContents
+    '''
+    def publish(self):
+        # set title
+        self.published_title = self.draft_title
+        self.navigation_link_name = self.draft_navigation_link_name
+
+        if self.published_version != self.draft_version:
+
+            self.published_version = self.draft_version
+            self.published_at = timezone.now()
+
+        self.save(published=True)
+        
+
+    def save(self, *args, **kwargs):
+
+        # indicates, if the save() command came from self.publish
+        published = kwargs.pop('published', False)
+        
+        # if only a new preview token is needed skip the publication protocol
+        disallow_new_version = kwargs.pop('disallow_new_version', False)
+
+        # first, check the slug - new slug if the title really changed
+        slug_base = LocalizedTemplateContent.objects.generate_slug_base(self.draft_title)
+        if not self.slug.startswith(slug_base):
+            old_slug = self.slug
+            self.slug = LocalizedTemplateContent.objects.generate_slug(self.draft_title)
+            new_slug = self.slug
+
+            # add to SlugTrail - make old links still work
+            slug_trail = SlugTrail(
+                old_slug = old_slug,
+                new_slug = new_slug,
+            )
+
+            slug_trail.save()
+
+        if not self.pk:
+
+            if self.language != self.template_content.app.primary_language:
+                master_ltc = self.template_content.get_localized(self.template_content.app.primary_language)
+                self.draft_version = master_ltc.draft_version
+
+        else:
+
+            if disallow_new_version == False and published == False:
+
+                # the localized_template_content has already been published. start new version
+                if self.published_version == self.draft_version:
+                    self.draft_version += 1
+                    self.translation_ready = False
+
+        super().save(*args, **kwargs)
+
+
 # if a slug is changed, remember the wold slug for a forward
 # e.g. a user changes the title of an entry
 class SlugTrail(models.Model):
@@ -520,22 +527,23 @@ class SlugTrail(models.Model):
 '''
 class FlagTree:
 
+    # flags is a TemplateContentFlags QuerySet
     def __init__(self, flags, language, max_levels=2, **kwargs):
         self.flags = flags.order_by('parent_flag')
         self.language = language
         self.flag_tree = {}
-        self.preview = kwargs.get('preview', False)
+        
+        self.app_state = kwargs.get('app_state', 'published')
 
         self.toplevel_count = 0
 
         # preview fetches localized_template_content.draft_title
         title_attr = 'published_title'
         navigation_link_attr = 'navigation_link_name'
-        if self.preview:
+        if self.app_state == 'preview':
             title_attr = 'draft_title'
             navigation_link_attr = 'draft_navigation_link_name'
 
-
         
         for flag in self.flags:
 
@@ -546,7 +554,7 @@ class FlagTree:
                 self.toplevel_count += 1
 
                 if flag.parent_flag:
-                    raise ValueError('navigation nesting not implemented')
+                    raise NotImplementedError('navigation nesting is not implemented yet')
 
                 localized_template_content = tree_entry['localized_template_content'] 
 
@@ -560,17 +568,18 @@ class FlagTree:
         # only return unpublished localized_template_contents if preview is set to True
         localized_template_content = flag.template_content.get_localized(self.language)
 
-        if self.preview == True or localized_template_content.published_version:
+        if localized_template_content:
+            if self.app_state == 'preview' or localized_template_content.published_version:
 
-            tree_entry = {
-                'localized_template_content' : localized_template_content,
-                'children' :[],
-                'in_app_link' : localized_template_content.in_app_link(preview=self.preview),
-                'navigation_link_name' : localized_template_content.navigation_link_name,
-            }
+                tree_entry = {
+                    'localized_template_content' : localized_template_content,
+                    'children' :[],
+                    'in_app_link' : localized_template_content.in_app_link(app_state=self.app_state),
+                    'navigation_link_name' : localized_template_content.navigation_link_name,
+                }
 
-            if self.preview:
-                tree_entry['navigation_link_name'] = localized_template_content.draft_navigation_link_name
+                if self.app_state == 'preview':
+                    tree_entry['navigation_link_name'] = localized_template_content.draft_navigation_link_name
 
         return tree_entry
         
@@ -590,9 +599,7 @@ class TemplateContentFlagsManager(models.Manager):
     # do not retrieve flags across apps
     def get_tree(self, app, flag, language, **kwargs):
 
-        preview = kwargs.get('preview', True)
-
-        theme_settings = app.get_online_content_settings(preview=preview)
+        theme_settings = app.get_online_content_settings()
         max_flag_levels = theme_settings.get('max_flag_levels', 2)
 
         flags = self.filter(template_content__app=app, flag=flag)
@@ -601,6 +608,7 @@ class TemplateContentFlagsManager(models.Manager):
 
         return flag_tree
     
+    
 class TemplateContentFlags(models.Model):
     template_content = models.ForeignKey(TemplateContent, on_delete=models.CASCADE)
     flag = models.CharField(max_length=255) # e.g. 'footer', 'nav', 'header'
@@ -700,9 +708,6 @@ class CMSMicroContent(models.Model):
     def get_locale_model(self):
         return globals()['Localized%s' % self._meta.model.__name__]
 
-    @classmethod
-    def get_publication_model(self):
-        return globals()[self._meta.model.__name__.replace('Draft', 'Published')]
 
     def get_content(self, language):
         lmc = self.get_localized(language)
@@ -734,7 +739,11 @@ class DraftCMSMicroContent(CMSMicroContent):
 
     objects = DraftMicroContentManager()
 
-    def get_form_field(self, widget_attrs, *args, **kwargs):
+    @classmethod
+    def get_publication_model(self):
+        return globals()[self._meta.model.__name__.replace('Draft', 'Published')]
+
+    def get_form_field(self, app, template_content, widget_attrs, *args, **kwargs):
         raise NotImplementedError('CMSMicroContent need a get_form_field method')
 
     def translation_complete(self, language):
@@ -844,19 +853,21 @@ class PublishedCMSMicroContent(CMSMicroContent):
 '''
 class LocalizedCMSMicroContentManager(models.Manager):
     
-    def create(self, draft_microcontent, language, content, creator, **kwargs):
+    def create(self, microcontent, language, content, creator, **kwargs):
 
-        localized_draft_microcontent = self.model(
+        # can be localized_draft_microcontent or localized_published_microcontent
+
+        localized_microcontent = self.model(
             language = language,
-            microcontent = draft_microcontent,
+            microcontent = microcontent,
             content = content,
             creator = creator,
             **kwargs
         )
 
-        localized_draft_microcontent.save()
+        localized_microcontent.save()
 
-        return localized_draft_microcontent
+        return localized_microcontent
 
 
 class LocalizedCMSMicroContent(models.Model):
@@ -901,7 +912,7 @@ class LocalizedCMSMicroContent(models.Model):
 '''
 class DraftTextMicroContent(DraftCMSMicroContent):
 
-    def get_form_field(self, widget_attrs, *args, **kwargs):
+    def get_form_field(self, app, template_content, widget_attrs, *args, **kwargs):
 
         widget = forms.Textarea
         if 'short' in args:
@@ -990,7 +1001,7 @@ class LocalizedPublishedTextMicroContent(LocalizedTextMicroContent):
 
 class DraftImageMicroContent(DraftCMSMicroContent):
 
-    def get_form_field(self, widget_attrs, *args, **kwargs):
+    def get_form_field(self, app, template_content, widget_attrs, *args, **kwargs):
 
         language = widget_attrs['language']
 
@@ -998,18 +1009,44 @@ class DraftImageMicroContent(DraftCMSMicroContent):
         if limc:
             widget_attrs['file'] = limc.content
 
-        licenced_url_name = 'upload_licenced_image'
-            
-        data_url_reverse_kwargs = {
+
+        data_url_kwargs = {
+            'app_uid' : app.uid,
+            'template_content_id' : template_content.id,
             'language' : language,
             'microcontent_category' : widget_attrs['data-microcontentcategory'],
-            'microcontent_type' : widget_attrs['data-microcontenttype'],
         }
-        widget_attrs['data-url'] = reverse('upload_image', kwargs=data_url_reverse_kwargs)
+
+        licenced_url_kwargs = data_url_kwargs.copy()
+
+        data_url_kwargs['microcontent_type'] = widget_attrs['data-microcontenttype']
+
+        delete_url = None
+
+        if self.pk:
+
+            licenced_url_kwargs['microcontent_id'] = self.pk
+            licenced_url = reverse('update_licenced_image', kwargs=licenced_url_kwargs)
+            
+            delete_kwargs = {
+                'app_uid' : app.uid,
+                'template_content_id' : template_content.id,
+                'language' : language,
+            }
+            delete_url = reverse('DeleteFileContent', kwargs=delete_kwargs)
+            
+        else:
+            licenced_url_kwargs['microcontent_type'] = widget_attrs['data-microcontenttype']
+            licenced_url = reverse('upload_licenced_image', kwargs=licenced_url_kwargs)
+            
+        data_url = reverse('upload_image', kwargs=data_url_kwargs)
+            
+        widget_attrs['data-url'] = data_url  
         widget_attrs['accept'] = 'image/*'
 
         form_field = forms.ImageField(widget=forms.FileInput(widget_attrs), **kwargs)
-        form_field.licenced_url_name = licenced_url_name
+        form_field.licenced_url = licenced_url
+        form_field.delete_url = delete_url
         return form_field
 
     # always falls back to primary language image
@@ -1022,7 +1059,7 @@ class DraftImageMicroContent(DraftCMSMicroContent):
         limc = self.get_localized(language)
 
         if not limc:
-            limc = LocalizedImageMicroContent.objects.create(self, language, content, editor)
+            limc = LocalizedDraftImageMicroContent.objects.create(self, language, content, editor)
         else:
         
             limc.last_modified_by = editor
